/// <summary>
/// Dynamicweb Customer Center Subscriber
/// </summary>
codeunit 50015 DynamicwebCustCenSubscriber
{
    EventSubscriberInstance = StaticAutomatic;

    var
        ElementHelper: Codeunit ElementHelper;
        DynamicwebXmlHelper: Codeunit DynamicwebXmlHelper;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnBeforeExecuteGetPdfForItemRequest', '', true, true)]
    procedure OnBeforeExecuteGetPdfForItemRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var response: BigText);
    var
        xmlRoot: XmlElement;
    begin
        // Modify request xml to your needs
        requestDoc.GetRoot(xmlRoot);
        xmlRoot.Attributes().Set('test', 'value');

        // Set stopExecution to cancel base application processing the request        
        //stopExecution := true;

        // If stopExecution set to true you can set custom response base64 pdf string
        // otherwise base app will return its output        
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnAfterExecuteGetPdfForItemRequest', '', true, true)]
    procedure OnAfterExecuteGetPdfForItemRequest(var requestDoc: XmlDocument; var response: BigText);
    begin
        //you can set custom response base64 pdf string
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnBeforeExecuteGetListRequest', '', true, true)]
    procedure OnBeforeExecuteGetListRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseDoc: XmlDocument);
    begin
        // Modify requestDoc request xml to your needs

        // Set stopExecution := true to cancel base application processing the request        
        // stopExecution := true;

        // If stopExecution set to true you can set custom xml for the response output        
        // otherwise base app will output its content        
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnAfterExecuteGetListRequest', '', true, true)]
    procedure OnAfterExecuteGetListRequest(var requestDoc: XmlDocument; var responseDoc: XmlDocument);
    begin
        // Add custom xml for the response output        
        ElementHelper.AddCustomElement('OnAfterExecuteGetListRequest', responseDoc);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnBeforeExecuteGetItemRequest', '', true, true)]
    procedure OnBeforeExecuteGetItemRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseDoc: XmlDocument);
    begin
        // Modify requestDoc request xml to your needs

        // Set stopExecution := true to cancel base application processing the request        
        // stopExecution := true;

        // If stopExecution set to true you can set custom xml for the response output        
        // otherwise base app will output its content 
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnAfterExecuteGetItemRequest', '', true, true)]
    procedure OnAfterExecuteGetItemRequest(var requestDoc: XmlDocument; var responseDoc: XmlDocument);
    begin
        // Add custom xml for the response output        
        ElementHelper.AddCustomElement('OnAfterExecuteGetItemRequest', responseDoc);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnBeforeExecuteGetCustomerStatementPfRequest', '', true, true)]
    procedure OnBeforeExecuteGetCustomerStatementPfRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var response: BigText);
    var
        xmlRoot: XmlElement;
    begin
        // Modify request xml to your needs
        requestDoc.GetRoot(xmlRoot);
        xmlRoot.Attributes().Set('test', 'value');

        // Set stopExecution to cancel base application processing the request        
        //stopExecution := true;

        // If stopExecution set to true you can set custom response base64 pdf string
        // otherwise base app will return its output   
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'OnAfterExecuteGetCustomerStatementPfRequest', '', true, true)]
    procedure OnAfterExecuteGetCustomerStatementPfRequest(var requestDoc: XmlDocument; var response: BigText);
    begin
        //you can set custom response base64 pdf string
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnBeforeAddOpenOrder', '', true, true)]
    procedure GetListOnBeforeAddOpenOrder(var salesHeader: Record "Sales Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnBeforeAddOpenOrder', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'customerNo', customer."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnAfterAddOpenOrder', '', true, true)]
    procedure GetListOnAfterAddOpenOrder(var salesHeader: Record "Sales Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnAfterAddOpenOrder', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesHeaderNo', salesHeader."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnBeforeAddInvoice', '', true, true)]
    procedure GetListOnBeforeAddInvoice(var salesInvoiceHeader: Record "Sales Invoice Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnBeforeAddInvoice', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'customerNo', customer."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnAfterAddInvoice', '', true, true)]
    procedure GetListOnAfterAddInvoice(var salesInvoiceHeader: Record "Sales Invoice Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnAfterAddInvoice', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesInvoiceHeaderNo', salesInvoiceHeader."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnBeforeAddCredit', '', true, true)]
    procedure GetListOnBeforeAddCredit(var salesCrMemoHeader: Record "Sales Cr.Memo Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnBeforeAddCredit', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'customerNo', customer."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnAfterAddCredit', '', true, true)]
    procedure GetListOnAfterAddCredit(var salesCrMemoHeader: Record "Sales Cr.Memo Header"; var customer: Record Customer; var outputNode: XmlNode);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetListOnAfterAddCredit', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesCrMemoHeaderNo', salesCrMemoHeader."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesHeader', '', true, true)]
    procedure GetItemOnBeforeAddSalesHeader(var salesHeader: Record "Sales Header"; var outputNode: XmlNode;
    var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesHeaderNo', salesHeader."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesHeader', '', true, true)]
    procedure GetItemOnAfterAddSalesHeader(var salesHeader: Record "Sales Header"; var outputNode: XmlNode;
    var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesHeaderNo', salesHeader."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesLine', '', true, true)]
    procedure GetItemOnBeforeAddSalesLine(var salesHeader: Record "Sales Header"; var salesLine: Record "Sales Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal; var cancelAdd: boolean);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesLineNo', salesLine."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesLine', '', true, true)]
    procedure GetItemOnAfterAddSalesLine(var salesHeader: Record "Sales Header"; var salesLine: Record "Sales Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesLineNo', salesLine."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesInvLine', '', true, true)]
    procedure GetItemOnBeforeAddSalesInvLine(var salesInvHeader: Record "Sales Invoice Header"; var salesInvLine: Record "Sales Invoice Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesInvLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesInvLineNo', salesInvLine."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesInvLine', '', true, true)]
    procedure GetItemOnAfterAddSalesInvLine(var salesInvHeader: Record "Sales Invoice Header"; var salesInvLine: Record "Sales Invoice Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesInvLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesInvLineNo', salesInvLine."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesInvHeader', '', true, true)]
    procedure GetItemOnBeforeAddSalesInvHeader(var salesInvHeader: Record "Sales Invoice Header"; var outputNode: XmlNode;
    var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesInvHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesInvHeaderNo', salesInvheader."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesInvHeader', '', true, true)]
    procedure GetItemOnAfterAddSalesInvHeader(var salesInvHeader: Record "Sales Invoice Header"; var outputNode: XmlNode;
    var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesInvHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesInvHeaderNo', salesInvheader."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesCrMemoHeader', '', true, true)]
    procedure GetItemOnBeforeAddSalesCrMemoHeader(var salesCrMemoHeader: Record "Sales Cr.Memo Header"; var outputNode: XmlNode;
        var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesCrMemoHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesCrMemoHeaderNo', salesCrMemoHeader."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesCrMemoHeader', '', true, true)]
    procedure GetItemOnAfterAddSalesCrMemoHeader(var salesCrMemoHeader: Record "Sales Cr.Memo Header"; var outputNode: XmlNode;
    var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesCrMemoHeader', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesCrMemoHeaderNo', salesCrMemoHeader."No.");
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnBeforeAddSalesCrMemoLine', '', true, true)]
    procedure GetItemOnBeforeAddSalesCrMemoLine(salesCrMemoHeader: Record "Sales Cr.Memo Header"; salesCrMemoLine: Record "Sales Cr.Memo Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnBeforeAddSalesCrMemoLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesCrMemoLineNo', salesCrMemoLine."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetItemOnAfterAddSalesCrMemoLine', '', true, true)]
    procedure GetItemOnAfterAddSalesCrMemoLine(salesCrMemoHeader: Record "Sales Cr.Memo Header"; salesCrMemoLine: Record "Sales Cr.Memo Line"; var outputNode: XmlNode;
    parentLineId: Text; var OrderPriceWithVAT: Decimal; var OrderPriceWithoutVAT: Decimal; var OrderPriceVAT: Decimal; var OrderSalesDiscount: Decimal);
    var
        xmlNode: XmlNode;
    begin
        DynamicwebXmlHelper.AddElement(outputNode, 'GetItemOnAfterAddSalesCrMemoLine', '', '', xmlNode);
        DynamicwebXmlHelper.AddField(xmlNode, 'salesCrMemoLineNo', salesCrMemoLine."No.");
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebCustomerCenPublisher, 'GetListOnAfterAddOrderXmlNode', '', true, true)]
    procedure GetListOnAfterAddOrderXmlNode(requsetType: Text; orderNo: Text; var outputNode: XmlNode);
    begin
    end;
}