/// <summary>
/// Represents global notifications subscribers
/// </summary>
codeunit 50010 DynamicwebGlobalSubscriber
{
    EventSubscriberInstance = StaticAutomatic;

    var
        DynamicwebXmlHelper: Codeunit DynamicwebXmlHelper;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebPublisher, 'OnBeforeExecuteRequest', '', true, true)]
    procedure OnBeforeExecuteRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseDoc: XmlDocument)
    var
        pXmlElement: XmlElement;
        xmlRoot: XmlElement;
        xmlRootNode: XmlNode;
    begin
        // Modify request xml to your needs
        //requestDoc.GetRoot(xmlRoot);
        //xmlRoot.Attributes().Set('test', 'value');

        // Set stopExecution to cancel base application processing the request        
        //stopExecution := true;

        // If stopExecution set to true you can Set custom xml for the response output        
        // otherwise base app will return its output
        //pXmlElement := XmlElement.Create('OnBeforeExecuteRequest');
        //pXmlElement.Add(XmlText.Create('extendable'));
        //XmlDocument.ReadFrom('<?xml version="1.0" encoding="utf-8" ?><tables/>', responseDoc);
        //responseDoc.GetRoot(xmlRoot);
        //xmlRoot.Add(pXmlElement);

        //Example: Adding a new entity with new fields
        //Returning Companies on GetCompanies request
        //If request has the xml format of <GetCompanies> then process Companies        
        DynamicwebXmlHelper.GetRootNode(requestDoc, xmlRootNode);
        if xmlrootnode.AsXmlElement().LocalName = 'GetCompanies' then begin
            // Set stopExecution to cancel base application processing the request
            // We will handle it by custom code    
            stopExecution := true;
            ProcessCompaniesRequest(responseDoc);
        end;
    end;

    local procedure ProcessCompaniesRequest(var responseDoc: XmlDocument)
    var
        company: Record Company;
        pXmlElement: XmlElement;
        xmlRootNode: XmlNode;
        companyTableXmlNode: XmlNode;
        companyNode: XmlNode;
    begin
        //Create xml with root <tables> element
        XmlDocument.ReadFrom('<?xml version="1.0" encoding="utf-8" ?><tables/>', responseDoc);
        if (company.FINDSET(false, false)) then begin
            responseDoc.GetRoot(pXmlElement);
            //get root node
            xmlRootNode := pXmlElement.AsXmlNode();
            //add <table tableName='Company'> xml node
            DynamicwebXmlHelper.AddElement(xmlRootNode, 'table', '', '', companyTableXmlNode);
            DynamicwebXmlHelper.AddAttribute(companyTableXmlNode, 'tableName', 'Company');
            repeat
                //add company fields to xml
                DynamicwebXmlHelper.AddElement(companyTableXmlNode, 'item', '', '', companyNode);
                DynamicwebXmlHelper.AddAttribute(companyNode, 'table', 'Company');
                DynamicwebXmlHelper.AddField(companyNode, 'CompanyId', company.Id);
                DynamicwebXmlHelper.AddField(companyNode, 'CompanyName', company.Name);
                DynamicwebXmlHelper.AddField(companyNode, 'CompanyDisplayName', company."Display Name");
                if company."Evaluation Company" then
                    DynamicwebXmlHelper.AddField(companyNode, 'CompanyEvaluation', 'true')
                else
                    DynamicwebXmlHelper.AddField(companyNode, 'CompanyEvaluation', 'false');
            until company.NEXT = 0;
        end;
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebPublisher, 'OnAfterExecuteXmlRequest', '', true, true)]
    procedure OnAfterExecuteXmlRequest(var requestDoc: XmlDocument; var responseDoc: XmlDocument)
    var
        pXmlElement: XmlElement;
        xmlRoot: XmlElement;
    begin
        // Add custom xml for the response output        
        pXmlElement := XmlElement.Create('OnAfterExecuteXmlRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseDoc.GetRoot(xmlRoot);
        xmlRoot.Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebPublisher, 'OnAfterExecutePdfRequest', '', true, true)]
    procedure OnAfterExecutePdfRequest(var requestDoc: XmlDocument; var response: BigText);
    begin
        Clear(response);
        response.AddText('OnAfterExecutePdfRequest: This is custom pdf string');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebPublisher, 'OnBeforeSendXmlResponse', '', true, true)]
    procedure OnBeforeSendXmlResponse(var requestDoc: XmlDocument; var responseDoc: XmlDocument);
    var
        pXmlElement: XmlElement;
        xmlRoot: XmlElement;
    begin
        // Add custom xml for the response output        
        pXmlElement := XmlElement.Create('OnBeforeSendXmlResponse');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseDoc.GetRoot(xmlRoot);
        xmlRoot.Add(pXmlElement);
    end;
}