codeunit 50013 DynamicwebUsersSubscriber
{
    EventSubscriberInstance = StaticAutomatic;

    var
        DynamicwebXmlHelper: Codeunit DynamicwebXmlHelper;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnBeforeGetCustomersRequest', '', true, true)]
    procedure OnBeforeGetCustomersRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseRootNode: XmlNode);
    var
        pXmlElement: XmlElement;
    begin
        // Modify requestDoc request xml to your needs

        // Set stopExecution := true to cancel base application processing the request        
        // stopExecution := true;

        // If stopExecution set to true you can Set custom xml for the response output        
        // otherwise base app will append its output to responseDoc
        pXmlElement := XmlElement.Create('OnBeforeGetCustomersRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseRootNode.AsXmlElement().Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAfterGetCustomersRequest', '', true, true)]
    procedure OnAfterGetCustomersRequest(var requestDoc: XmlDocument; var responseRootNode: XmlNode);
    begin
        // Add custom xml for the response output        
        AddCustomElement(responseRootNode, 'OnAfterGetCustomersRequest');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnBeforeGetContactsRequest', '', true, true)]
    procedure OnBeforeGetContactsRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseRootNode: XmlNode);
    var
        pXmlElement: XmlElement;
    begin
        // Modify requestDoc request xml to your needs

        // Set stopExecution := true to cancel base application processing the request        
        // stopExecution := true;

        // If stopExecution set to true you can Set custom xml for the response output        
        // otherwise base app will append its output to responseDoc
        pXmlElement := XmlElement.Create('OnBeforeGetContactsRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseRootNode.AsXmlElement().Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAfterGetContactsRequest', '', true, true)]
    procedure OnAfterGetContactsRequest(var requestDoc: XmlDocument; var responseRootNode: XmlNode);
    begin
        // Add custom xml for the response output        
        AddCustomElement(responseRootNode, 'OnAfterGetContactsRequest');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnBeforeGetAddressesRequest', '', true, true)]
    procedure OnBeforeGetAddressesRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseRootNode: XmlNode);
    var
        pXmlElement: XmlElement;
    begin
        // Modify requestDoc request xml to your needs

        // Set stopExecution := true to cancel base application processing the request        
        // stopExecution := true;

        // If stopExecution set to true you can Set custom xml for the response output        
        // otherwise base app will append its output to responseDoc
        pXmlElement := XmlElement.Create('OnBeforeGetAddressesRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseRootNode.AsXmlElement().Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAfterGetAddressesRequest', '', true, true)]
    procedure OnAfterGetAddressesRequest(var requestDoc: XmlDocument; var responseRootNode: XmlNode);
    begin
        // Add custom xml for the response output        
        AddCustomElement(responseRootNode, 'OnAfterGetAddressesRequest');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnBeforeExecutePutUsersRequest', '', true, true)]
    procedure OnBeforeExecutePutUsersRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseDoc: XmlDocument);
    var
        pXmlElement: XmlElement;
        xmlRoot: XmlElement;
    begin
        // Modify request xml to your needs
        requestDoc.GetRoot(xmlRoot);
        xmlRoot.Attributes().Set('test', 'value');

        // // Set stopExecution to cancel base application processing the request        
        // stopExecution := true;

        // // If stopExecution set to true you can Set custom xml for the response output        
        // otherwise base app will append its output to responseDoc
        pXmlElement := XmlElement.Create('OnBeforeExecutePutUsersRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseDoc.GetRoot(xmlRoot);
        xmlRoot.Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAfterExecutePutUsersRequest', '', true, true)]
    procedure OnAfterExecutePutUsersRequest(var requestDoc: XmlDocument; var responseDoc: XmlDocument);
    var
        pXmlElement: XmlElement;
        xmlRoot: XmlElement;
    begin
        // Add custom xml for the response output        
        pXmlElement := XmlElement.Create('OnAfterExecutePutUsersRequest');
        pXmlElement.Add(XmlText.Create('extendable'));
        responseDoc.GetRoot(xmlRoot);
        xmlRoot.Add(pXmlElement);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAddGroupXmlNode', '', true, true)]
    procedure OnAddGroupXmlNode(var groupNode: XmlNode; groupName: Text);
    var
        tableXmlElement: XmlElement;
    begin
        //Instead of creating one group 'Customers' create two groups 'Customers1' and 'Customers2'.
        if groupName = 'Customers' then begin
            groupNode.GetParent(tableXmlElement);//get <table> element                
            groupNode.Remove(); //Remove 'Customers' <item table="AccessUserGroup"> node            
            //Add Customers1 
            AddGroupXmlNode(tableXmlElement, 'Customers1');
            //Add Customers2
            AddGroupXmlNode(tableXmlElement, 'Customers2');
        end;

        //Instead of creating one group 'Sales' create two groups 'Senior Sales' and 'Junior Sales'
        if groupName = 'Sales' then begin
            groupNode.GetParent(tableXmlElement);//get <table> element                
            groupNode.Remove(); //Remove 'Sales' <item table="AccessUserGroup"> node            
            AddGroupXmlNode(tableXmlElement, 'Senior Sales');
            AddGroupXmlNode(tableXmlElement, 'Junior Sales');
        end;
    end;

    /// <summary>
    /// Creates and adds AccessUserGroup <item /> to the <table tableName="AccessUserGroup" /> xml
    /// </summary>
    /// <param name="tableXmlElement">table xml element</param>
    /// <param name="groupName">group name</param>
    local procedure AddGroupXmlNode(var tableXmlElement: XmlElement; groupName: Text)
    var
        item: XmlElement;
        node: XmlNode;
    begin
        //add <item> to the <table> xml structure that looks like that:
        //   <table tableName="AccessUserGroup">            
        //     <item table="AccessUserGroup">
        //       <column columnName="AccessGroupName"><![CDATA[groupName]]></column>
        //       <column columnName="AccessGroupGroupName"><![CDATA[groupName]]></column>
        //     </item>
        //   </table>

        //<item>
        item := Xmlelement.Create('item');
        //<item table="AccessUserGroup">
        item.Attributes().Set('table', 'AccessUserGroup');
        node := item.AsXmlNode();
        //Add <column columnName="AccessGroupName"><![CDATA[groupName]]></column>
        DynamicwebXmlHelper.AddField(node, 'AccessGroupName', groupName);
        //Add <column columnName="AccessGroupGroupName"><![CDATA[groupName]]></column>
        DynamicwebXmlHelper.AddField(node, 'AccessGroupGroupName', groupName);
        //Add <item> to parent the <table> xml
        tableXmlElement.Add(item);
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAddContactXmlNode', '', true, true)]
    procedure OnAddContactXmlNode(var contactNode: XmlNode; contact: Record Contact; contactCustomer: Record Customer);
    var
        node: XmlNode;
        element: XmlElement;
    begin
        //Add custom field
        DynamicwebXmlHelper.AddField(contactNode, 'AccessUserEmail2', contact."E-Mail 2");

        //Update user groups field with custom group name
        //Instead of placing user to 'Customers' group put it to 'Customers1' or 'Customers2'
        if contactNode.SelectSingleNode('column[@columnName=''AccessUserGroups'']', node) then begin
            element := node.AsXmlElement();
            if (element.InnerText = 'Customers') then begin
                element.RemoveNodes();
                element.Add(XmlCData.Create(GetContactGroup(contact)));
            end;
        end;
    end;

    //Get group name based on if contact name contains 'a' letter
    local procedure GetContactGroup(contact: Record Contact): Text
    begin
        if contact.Name.Contains('a') then
            exit('Customers1')
        else
            exit('Customers2');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAddSalesPersonXmlNode', '', true, true)]
    procedure OnAddSalesPersonXmlNode(var salesPersonXmlNode: XmlNode; salesPerson: Record "Salesperson/Purchaser");
    var
        node: XmlNode;
        element: XmlElement;
    begin
        //Add custom field
        DynamicwebXmlHelper.AddField(salesPersonXmlNode, 'AccessUserPhonePriv', 'PhoneFromAnotherSystem');

        //Update user groups field with custom group name
        //Instead of placing user to 'Sales' group put it to 'Senior Sales' or 'Junior Sales'
        if salesPersonXmlNode.SelectSingleNode('column[@columnName=''AccessUserGroups'']', node) then begin
            element := node.AsXmlElement();
            if (element.InnerText = 'Sales') then begin
                element.RemoveNodes();
                element.Add(XmlCData.Create(GetSalespersonGroup(salesPerson)));
            end;
        end;
    end;

    //Get group name based on "Commission %"
    local procedure GetSalespersonGroup(salesPerson: Record "Salesperson/Purchaser"): Text
    begin
        if salesPerson."Commission %" >= 5 then
            exit('Senior Sales')
        else
            exit('Junior Sales');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAddImpersonationXmlNode', '', true, true)]
    procedure OnAddImpersonationXmlNode(var impersonationXmlNode: XmlNode; contact: Record Contact; salesPerson: Record "Salesperson/Purchaser");
    var
        tableXmlElement: XmlElement;
        item: XmlElement;
        node: XmlNode;
        externalImpersonatorId: Text;
    begin
        //Override impersonations: salesPerson - customer contact relation comes from another system                
        externalImpersonatorId := GetExternalContactImpersonator(contact);
        //if external Impersonator Id is found override salesPerson - customer contact relation
        if externalImpersonatorId <> '' then begin
            impersonationXmlNode.GetParent(tableXmlElement);//get <table> element                
            impersonationXmlNode.Remove(); //Remove <item table="AccessUserSecondaryRelation"> node   

            //<item>
            item := Xmlelement.Create('item');
            //<item table="AccessUserSecondaryRelation">
            item.Attributes().Set('table', 'AccessUserSecondaryRelation');
            node := item.AsXmlNode();
            //Add <column columnName="AccessUserSecondaryRelationUserId"><![CDATA[]]></column>
            DynamicwebXmlHelper.AddField(node, 'AccessUserSecondaryRelationUserId', externalImpersonatorId);
            //Add <column columnName="AccessUserSecondaryRelationSecondaryUserId"><![CDATA[]]></column>
            DynamicwebXmlHelper.AddField(node, 'AccessUserSecondaryRelationSecondaryUserId', Contact."No.");
            //Add <item> to parent the <table> xml
            tableXmlElement.Add(item);
        end;
    end;

    /// <summary>
    /// This function will return contact impersonator id from external system
    /// </summary>
    /// <param name="contact">contact</param>
    /// <returns>contact impersonator id if found, otherwise empty string</returns>
    local procedure GetExternalContactImpersonator(contact: Record Contact): Text
    begin
        if contact.Name.Contains('a') then
            exit('ExternalImpersonatorId')
        else
            exit('');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnAddAddressXmlNode', '', true, true)]
    procedure OnAddAddressXmlNode(var addressXmlNode: XmlNode; contact: Record Contact);
    var
    begin
        //Add custom field                
        DynamicwebXmlHelper.AddField(addressXmlNode, 'AccessUserAddress3', contact.Address + ' ' + contact."Address 2");
    end;

    local procedure AddCustomElement(var responseRootNode: XmlNode; elementName: Text);
    var
        pXmlElement: XmlElement;
    begin
        // Add custom xml for the response output        
        pXmlElement := XmlElement.Create(elementName);
        pXmlElement.Add(XmlText.Create('extendable'));
        responseRootNode.AsXmlElement().Add(pXmlElement);
    end;


    /// <summary>
    /// This event fires at /GetEcomData/tables/Customers/@type='all' request
    /// before customer records are enumerated
    /// </summary>
    /// <param name="customer">customer records</param>
    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebUsersPublisher, 'OnBeforeFindCustomers', '', true, true)]
    procedure OnBeforeFindCustomers(var customer: Record Customer);
    begin
        //you can filter your customer records here
        //customer.SetFilter();
    end;
}